@@ -38,16 +38,17 @@
 #include "gdata-calendar-calendar.h"
 #include "gdata-private.h"
 #include "gdata-service.h"
-#include "gdata-gdata.h"
 #include "gdata-parser.h"
 #include "gdata-types.h"
+#include "gdata-access-handler.h"
 
+static void gdata_calendar_calendar_access_handler_init (GDataAccessHandlerIface *iface);
 static void gdata_calendar_calendar_finalize (GObject *object);
 static void gdata_calendar_calendar_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_calendar_calendar_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
-static void get_xml (GDataEntry *entry, GString *xml_string);
-static gboolean parse_xml (GDataEntry *entry, xmlDoc *doc, xmlNode *node, GError **error);
-static void get_namespaces (GDataEntry *entry, GHashTable *namespaces);
+static void get_xml (GDataParsable *parsable, GString *xml_string);
+static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
+static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
 
 struct _GDataCalendarCalendarPrivate {
 	gchar *timezone;
@@ -70,14 +71,15 @@ enum {
 	PROP_EDITED
 };
 
-G_DEFINE_TYPE (GDataCalendarCalendar, gdata_calendar_calendar, GDATA_TYPE_ENTRY)
+G_DEFINE_TYPE_WITH_CODE (GDataCalendarCalendar, gdata_calendar_calendar, GDATA_TYPE_ENTRY,
+			 G_IMPLEMENT_INTERFACE (GDATA_TYPE_ACCESS_HANDLER, gdata_calendar_calendar_access_handler_init))
 #define GDATA_CALENDAR_CALENDAR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_CALENDAR_CALENDAR, GDataCalendarCalendarPrivate))
 
 static void
 gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-	GDataEntryClass *entry_class = GDATA_ENTRY_CLASS (klass);
+	GDataParsableClass *parsable_class = GDATA_PARSABLE_CLASS (klass);
 
 	g_type_class_add_private (klass, sizeof (GDataCalendarCalendarPrivate));
 
@@ -85,9 +87,9 @@ gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 	gobject_class->get_property = gdata_calendar_calendar_get_property;
 	gobject_class->finalize = gdata_calendar_calendar_finalize;
 
-	entry_class->get_xml = get_xml;
-	entry_class->parse_xml = parse_xml;
-	entry_class->get_namespaces = get_namespaces;
+	parsable_class->parse_xml = parse_xml;
+	parsable_class->get_xml = get_xml;
+	parsable_class->get_namespaces = get_namespaces;
 
 	/**
 	 * GDataCalendarCalendar:timezone:
@@ -180,6 +182,18 @@ gdata_calendar_calendar_init (GDataCalendarCalendar *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_CALENDAR_CALENDAR, GDataCalendarCalendarPrivate);
 }
 
+static gboolean
+is_owner_rule (GDataAccessRule *rule)
+{
+	return (strcmp (gdata_access_rule_get_role (rule), "http://schemas.google.com/gCal/2005#owner") == 0) ? TRUE : FALSE;
+}
+
+static void
+gdata_calendar_calendar_access_handler_init (GDataAccessHandlerIface *iface)
+{
+	iface->is_owner_rule = is_owner_rule;
+}
+
 static void
 gdata_calendar_calendar_finalize (GObject *object)
 {
@@ -265,53 +279,36 @@ gdata_calendar_calendar_new (const gchar *id)
 	return g_object_new (GDATA_TYPE_CALENDAR_CALENDAR, "id", id, NULL);
 }
 
-/**
- * gdata_calendar_calendar_new_from_xml:
- * @xml: an XML string
- * @length: the length in characters of @xml, or %-1
- * @error: a #GError, or %NULL
- *
- * Creates a new #GDataCalendarCalendar from an XML string. If @length is %-1, the length of
- * the string will be calculated.
- *
- * Errors from #GDataParserError can be returned if problems are found in the XML.
- *
- * Return value: a new #GDataCalendarCalendar, or %NULL; unref with g_object_unref()
- **/
-GDataCalendarCalendar *
-gdata_calendar_calendar_new_from_xml (const gchar *xml, gint length, GError **error)
-{
-	return GDATA_CALENDAR_CALENDAR (_gdata_entry_new_from_xml (GDATA_TYPE_CALENDAR_CALENDAR, xml, length, error));
-}
-
 static gboolean
-parse_xml (GDataEntry *entry, xmlDoc *doc, xmlNode *node, GError **error)
+parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
-	GDataCalendarCalendar *self = GDATA_CALENDAR_CALENDAR (entry);
+	GDataCalendarCalendar *self;
 
-	g_return_val_if_fail (GDATA_IS_CALENDAR_CALENDAR (self), FALSE);
+	g_return_val_if_fail (GDATA_IS_CALENDAR_CALENDAR (parsable), FALSE);
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (node != NULL, FALSE);
 
+	self = GDATA_CALENDAR_CALENDAR (parsable);
+
 	if (xmlStrcmp (node->name, (xmlChar*) "timezone") == 0) {
 		/* gCal:timezone */
 		xmlChar *_timezone = xmlGetProp (node, (xmlChar*) "value");
 		if (_timezone == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:timezone", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_calendar_set_timezone (self, (gchar*) _timezone);
 		xmlFree (_timezone);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "timesCleaned") == 0) {
 		/* gCal:timesCleaned */
 		xmlChar *times_cleaned = xmlGetProp (node, (xmlChar*) "value");
 		if (times_cleaned == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:timesCleaned", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		self->priv->times_cleaned = strtoul ((gchar*) times_cleaned, NULL, 10);
 		xmlFree (times_cleaned);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "hidden") == 0) {
 		/* gCal:hidden */
 		xmlChar *hidden = xmlGetProp (node, (xmlChar*) "value");
 		if (hidden == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:hidden", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_calendar_set_is_hidden (self, (xmlStrcmp (hidden, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 		xmlFree (hidden);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "color") == 0) {
@@ -321,13 +318,18 @@ parse_xml (GDataEntry *entry, xmlDoc *doc, xmlNode *node, GError **error)
 
 		value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:color", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		if (gdata_color_from_hexadecimal ((gchar*) value, &colour) == FALSE) {
 			/* Error */
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-				     _("A <%s>'s <%s> element (\"%s\") was not in hexadecimal RGB format."),
-				     "entry", "gCal:color", value);
+				     /* Translators: the first parameter is the name of an XML element (including the angle brackets ("<" and ">"),
+				      * and the second parameter is the erroneous value (which was not in hexadecimal RGB format).
+				      *
+				      * For example:
+				      *  The content of a <entry/gCal:color> element ("00FG56") was not in hexadecimal RGB format. */
+				     _("The content of a %s element (\"%s\") was not in hexadecimal RGB format."), "<entry/gCal:color>", value);
 			xmlFree (value);
+
 			return FALSE;
 		}
 
@@ -337,25 +339,25 @@ parse_xml (GDataEntry *entry, xmlDoc *doc, xmlNode *node, GError **error)
 		/* gCal:selected */
 		xmlChar *selected = xmlGetProp (node, (xmlChar*) "value");
 		if (selected == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:selected", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_calendar_set_is_selected (self, (xmlStrcmp (selected, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 		xmlFree (selected);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "accesslevel") == 0) {
 		/* gCal:accesslevel */
 		self->priv->access_level = (gchar*) xmlGetProp (node, (xmlChar*) "value");
 		if (self->priv->access_level == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:accesslevel", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
 		/* app:edited */
-		xmlChar *edited = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
 			/* Error */
-			gdata_parser_error_not_iso8601_format ("app:edited", "entry", (gchar*) edited, error);
+			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
 			xmlFree (edited);
 			return FALSE;
 		}
 		xmlFree (edited);
-	} else if (GDATA_ENTRY_CLASS (gdata_calendar_calendar_parent_class)->parse_xml (entry, doc, node, error) == FALSE) {
+	} else if (GDATA_PARSABLE_CLASS (gdata_calendar_calendar_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
 		/* Error! */
 		return FALSE;
 	}
@@ -364,31 +366,38 @@ parse_xml (GDataEntry *entry, xmlDoc *doc, xmlNode *node, GError **error)
 }
 
 static void
-get_xml (GDataEntry *entry, GString *xml_string)
+get_xml (GDataParsable *parsable, GString *xml_string)
 {
-	GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR (entry)->priv;
+	gchar *colour;
+	GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR (parsable)->priv;
 
 	/* Chain up to the parent class */
-	GDATA_ENTRY_CLASS (gdata_calendar_calendar_parent_class)->get_xml (entry, xml_string);
+	GDATA_PARSABLE_CLASS (gdata_calendar_calendar_parent_class)->get_xml (parsable, xml_string);
 
 	/* Add all the Calendar-specific XML */
 	if (priv->timezone != NULL)
-		g_string_append_printf (xml_string, "<gCal:timezone value='%s'/>", priv->timezone);
+		gdata_parser_string_append_escaped (xml_string, "<gCal:timezone value='", priv->timezone, "'/>");
 
-	/* TODO:
-	 * - Finish supporting all tags
-	 * - Check all tags here are valid for insertions and updates
-	 * - Check things are escaped (or not) as appropriate
-	 * - Write a function to encapsulate g_markup_escape_text and
-	 *   g_string_append_printf to reduce the number of allocations
-	 */
+	if (priv->is_hidden == TRUE)
+		g_string_append (xml_string, "<gCal:hidden value='true'/>");
+	else
+		g_string_append (xml_string, "<gCal:hidden value='false'/>");
+
+	colour = gdata_color_to_hexadecimal (&(priv->colour));
+	g_string_append_printf (xml_string, "<gCal:color value='%s'/>", colour);
+	g_free (colour);
+
+	if (priv->is_selected == TRUE)
+		g_string_append (xml_string, "<gCal:selected value='true'/>");
+	else
+		g_string_append (xml_string, "<gCal:selected value='false'/>");
 }
 
 static void
-get_namespaces (GDataEntry *entry, GHashTable *namespaces)
+get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 {
 	/* Chain up to the parent class */
-	GDATA_ENTRY_CLASS (gdata_calendar_calendar_parent_class)->get_namespaces (entry, namespaces);
+	GDATA_PARSABLE_CLASS (gdata_calendar_calendar_parent_class)->get_namespaces (parsable, namespaces);
 
 	g_hash_table_insert (namespaces, (gchar*) "gCal", (gchar*) "http://schemas.google.com/gCal/2005");
 	g_hash_table_insert (namespaces, (gchar*) "app", (gchar*) "http://www.w3.org/2007/app");
